// Flip-Pal Toolbox Common thumbnail functions
// 2013 Jan 22 - Jim Moy, jim.moy@couragent.net

function centerImage( img, maxsize ) {
	// With CSS styling on the image, a maxsize x maxsize image is properly centered.
	// If the loaded image is smaller than maxsize x maxsize, adjust its margins so the
	// image's center is in the same place.
	img.css( "margin-top", Math.round( (maxsize - img.height()) / 2 ) );
	img.css( "margin-left", Math.round( (maxsize - img.width()) / 2 ) );
}

function getSelectedThumbPath() {
	var sel = $("#grid").find( ".thumbselect" );
	if( sel.length != 1 ) {
		return "";
	} else {
		return thumbURLtoThumbPath( unescape( sel[0].src ));
	}
}

function thumbURLtoImagePath(str) {
	var thumb = str.replace(/^file.*?([A-Z])/,"$1");
	thumb = thumb.replace(/(.+)\?ts=[0-9]*/,"$1");
	var img = thumb.replace(/\.thumb\//,"");
	if( navigator.platform.indexOf( "Mac" ) != -1 ) {
		return "/" + img;
	} else {
		return img;
	}
}
function thumbURLtoThumbPath(str) {
	var thumb = str.replace(/^file.*?([A-Z])/,"$1");
	thumb = thumb.replace(/(.+)\?ts=[0-9]*/,"$1");
	if( navigator.platform.indexOf( "Mac" ) != -1 ) {
		return "/" + thumb;
	} else {
		return thumb;
	}
}
function setImageOffsets() {
	$('div.imbox').each( function() {
		var $this = $(this);
		var boxHeight = $this.height();
		$this.find('img').each( function() {
			var $img = $(this);
			var height = (boxHeight - $img.height())/2;
			$img.css( "margin-top", height + "px" );
		});
	});
}

function bindImageSingleSelection() {
	$('.imcell').click( function( event ) {
		var thumb = $(this).find('img.thumb');
		if( thumb.length > 0 ) {
			thumb.toggleClass( 'thumbselect' );
			if( thumb.hasClass( 'thumbselect' )) {
				if( window.flipLastSelected != -1) {
					$('#imtbl').find('.imcell').eq( window.flipLastSelected ).find( '.thumb' ).toggleClass( 'thumbselect' );
				}
				window.flipLastSelected = tableCellIndex( $(this) );
			} else {
				window.flipLastSelected = -1;
			}
		}
	});
}
function bindImageClicks() {
	$(".imcell").click( function( event ) {
		if( event.shiftKey && window.flipLastSelected != -1 ) {
			extendSelection( $(this) );
		} else {
			var thumb = $(this).find("img.thumb");
			if( thumb.length > 0 ) {
				thumb.toggleClass( "thumbselect" );
				if( thumb.hasClass( "thumbselect" )) {
					window.flipLastSelected = tableCellIndex( $(this) );
				}
			}
		}
	});
}
function extendSelection( cell ) {
	var index = tableCellIndex( cell );
	var start = -1;
	var end = -1;
	// flipLastSelected is the index of the last clicked on cell that
	// was toggled on
	if( index < window.flipLastSelected ) {
		start = index;
		end = window.flipLastSelected;
	}
	else {
		start = window.flipLastSelected;
		end = index;
	}
	if( start != -1 && end != -1 ) {
		var cellList = $('#imtbl').find('.imcell');
		for( var i = start; i <= end; i++ ) {
			var child = cellList.eq( i );
			var thumb = child.find( '.thumb' );
			thumb.addClass( 'thumbselect' );
		}
	}
}
function tableCellIndex( cell ) {
	// Offset cell index by the number of rows times the images in the row
	var rowList = $('#imtbl').children();
	var rowLength = rowList.eq( 0 ).find( '.imcell' ).length;
	var startRow = cell.parent().index(); // <tr><td/></tr>
	return startRow * rowLength + cell.index();
}
function thumbDisplayName( thumbPath ) {
	var base = baseName( thumbPath );
	var name = base.substr( 0, base.length - 4 ); // lop off .jpg
	if( name.length > 14 ) {
		return name.substr( 0, 6 ) + "..." + name.substr( name.length - 6 );
	}
	else {
		return name;
	}
}
function thumbCallbackFinished() {
	if( window.flipIsSingleSelectPage === true ) {
		bindImageSingleSelection();
	} else {
		bindImageClicks();
	}
	window.setTimeout( function() {
		setImageOffsets();
	}, 500 ); // Interesting any delay is needed, rendering supposed to be done.
	$.unblockUI();
	window.flipIsUIBlocked = false;
}
function thumbCallback( imgPath, thumbPath, size, finished, imgCount ) {
	if( window.flipRowIndex == 0 ) {
		window.flipCurrentRow = '\n<tr class="imrow">\n'; // new row
	}
	var dispName = thumbDisplayName( thumbPath );
	var cell = '\n\
<td class="imcell">\n\
	<div class="imbox">\n\
		<a target="_blank" href="" onclick="return false;">\n\
			<img class="thumb" src="file:///' + thumbPath + '?ts=' + new Date().getTime() + '" alt="Scan"/></a>\n\
	</div>\n\
	<div class="desc">' + dispName + '</div>\n\
</td>\n';
	window.flipCurrentRow = window.flipCurrentRow + cell;
	window.flipRowIndex++;
	if( window.flipRowIndex >= 6 || finished ) {
		window.flipCurrentRow = window.flipCurrentRow + '\n</tr>\n';
		$('#imtbl').append( window.flipCurrentRow );
		window.flipRowIndex = 0;
	}
	if( finished ) {
		thumbCallbackFinished();
	}
}
function flipStitchThumbsCallback( imgPath, thumbPath, size, finished, imgCount ) {
	if( $('#please-wait')) {
		$('#please-wait').hide( 'fast' );
	}
	if( imgCount == 0 ) {
		$('#imageui').hide('fast');
		$('#noimages').show('fast');
	} else {
		$('#noimages').hide('fast');
		$('#imageui').show('fast');
		if( imgCount == -1 ) { // flipstitchthumbs was canceled
			thumbCallbackFinished();
		} else {
			if( !window.flipIsUIBlocked ) {
				var wheel = '<img src="images/progress.gif" />';
				var cancelbtn = '<a onclick="cancelFlipStitchThumbs()" style="position:relative;top:20px;left:0px;">Cancel</a>';
				$.blockUI( {
					message: wheel + '<br>' + cancelbtn,
					css: { border: 'none', background: 'none' }
				});
				window.flipIsUIBlocked = true;
			}
			thumbCallback( imgPath, thumbPath, size, finished, imgCount );
		}
	}
}

function flipChooseDirCallback( arg ) {
	if( arg ) {
		// escaped for python command line (doesn't matter on Mac)
		var escaped = arg.replace( /\\/g, "/" );
		commandLine(['setdir', escaped], 'flipSetDirCallback' );
	}
}
function flipSetDirCallback( arg ) {
	var obj = $.parseJSON( arg )
	if( obj.status == "ok"  ) {
		if( window.flipIsSingleSelectPage === true ) {
			scanCard( true );
		} else {
			scanCard();
		}
	}
}

function emptyCallback( arg ) {}

function cancelFlipStitchThumbs() {
	commandLine( ['stitchthumbscanceled', '1'], 'emptyCallback' )
}

function startFlipStitchThumbsCallback() {
	app.sendMessage( 'flipstitchthumbs' );
}

if( typeof app != 'undefined' ) {
	app.setMessageCallback( 'flipstitchthumbs', function( name, args ) {
		// var imgPath = args[0]; // U:/DCIM/100_SCAN/SCAN0482.JPG
		// var thumbPath = args[1]; // U:/DCIM/100_SCAN/.thumb/SCAN0482.JPG
		// var size = args[2]; // 600x400
		// var finished = args[3]; // (boolean)
		// var imgCount = args[4]; // (int)
		flipStitchThumbsCallback( args[0], args[1], args[2], args[3], args[4] );
	});
	app.setMessageCallback( 'flipchoosedir', function( name, args ) {
		//var path = args[0]; (path to selected directory)
		flipChooseDirCallback( args[0] );
	});
	$.blockUI.defaults.overlayCSS.backgroundColor = '#fff';
	$.blockUI.defaults.overlayCSS.opacity = 0.90;
	function scanCard( isSingleSelectPage ) {
		window.flipRowIndex = 0;
		window.flipCurrentRow = "";
		window.flipLastSelected = -1;
		if( typeof( isSingleSelectPage ) === 'undefined' ) {
			window.flipIsSingleSelectPage = false;
		} else {
			window.flipIsSingleSelectPage = true;
		}
		window.flipIsUIBlocked = false;
		$('#imtbl').empty();
		commandLine( ['stitchthumbscanceled', '0'], 'startFlipStitchThumbsCallback' )
	}
}

