import httplib, os, platform, string, sys, time, traceback
import simplejson as json
import urllib2
import sys

from fpcommon import is_mac, is_windows
from os.path import *
from os import remove, rename, close, listdir, mkdir
from tempfile import mkstemp
from fnmatch import fnmatch
from urllib import urlopen, urlretrieve
from string import ascii_uppercase, digits
from random import choice, random

# resource\scripts\lib\ contains Evernote SDK and related modules.
# sys.path[0] is the directory containing the script used to invoke the
# interpreter, which should be resource\scripts\
libpath = normpath( join( sys.path[0], 'lib' ) )
sys.path.append( libpath )
from evernote_upload import EvernoteUploader

# A story is an image created by the record tab. They're stored in
# a sub-directory relative to the path of the images they were created from
STORY_SUBDIR = "StoryScans"

# Assume python 2.5, since 2.5.1 comes on OS-X Leopard, and we have a nice compact
# distribution called SAP on Windows which has version 2.5.2.

def run_process( cmd ):
	from subprocess import Popen, STDOUT
	from tempfile import NamedTemporaryFile
	f = NamedTemporaryFile()
	if is_mac():
		p = Popen( cmd, stdout=f, stderr=STDOUT, shell=True )
	elif is_windows():
		p = Popen( cmd, stdout=f, stderr=STDOUT )
	p.wait()
	f.seek( 0 )
	return f.read()

# PUT and POST requests for Dropbox/Picasa uploads
def upload_request( url, data, headers, put ):
	opener = urllib2.build_opener( urllib2.HTTPHandler )
	request = urllib2.Request( url, data=data )
	for k in headers:
		request.add_header( k, headers[k] )
	if put:
		request.get_method = lambda: 'PUT'
	return opener.open( request )

def picasa_get_post_data( imgdata, filename ):
	imgxml = '<entry xmlns="http://www.w3.org/2005/Atom">'
	imgxml += '<title>%s</title>' % filename
	imgxml += '<summary></summary>'
	imgxml += '<category scheme="http://schemas.google.com/g/2005#kind" term="http://schemas.google.com/photos/2007#photo"/>'
	imgxml += '</entry>'
	boundary = str( int( random() * 1e10 ) )
	parts = []
	parts.append( '' )
	parts.append( 'Media multipart posting' )
	parts.append( '--' + boundary )
	parts.append( 'Content-Type: application/atom+xml' )
	parts.append( '' )
	parts.append( imgxml )
	parts.append( '--' + boundary )
	parts.append( 'Content-Type: image/jpeg' )
	parts.append( '' )
	parts.append( imgdata )
	parts.append( '--' + boundary + '--' )
	data = '\r\n'.join( parts )
	return data, boundary

def get_image( filepath ):
	filename = basename( filepath )
	f = open( filepath, 'rb' )
	filecontents = str( f.read() )
	f.close()
	return filename, filecontents

class FlipPalCommand:

	def __init__( self, inp, out, path, interact ):
		self.input = inp
		self.output = out
		self.rsrc_path = path
		self.prompt = interact
		self.vol_dir = "/DCIM/100_SCAN"
		self.list_dir = None
		self.photoloom = None
		self.backup = None
		self.update = None
		self.rratio = None
		self.gratio = None
		self.bratio = None
		self.stitchthumbscanceled = False
		self.evernote_uploader = None

	def command( self, args ):
		json.dump( self.run_command( args ), self.output, indent = 2 )
		self.output.write( "\n" )
		if not self.prompt:
			self.output.write( "end\n" )
		self.output.flush()

	def run_command( self, args ):
		try:
			if args[0] == 'startup':
				result = self.startup()
			elif args[0] == 'rawaudiopath':
				return { 'status' : 'ok', 'path' : join( self.rsrc_path, 'recording.raw' ) }
			elif args[0] == 'saveproductkey':
				result = self.save_product_key( args[1] )
			elif args[0] == 'productkeypath':
				return { 'status' : 'ok', 'path' : join( self.rsrc_path, 'productkey' ) }
			elif args[0] == 'sysopen':
				result = self.system_open( args[1] )
			elif args[0] == 'rsrcopen':
				result = self.resource_open( args[1] )
			elif args[0] == 'flipopen':
				result = self.flip_open()
			elif args[0] == 'check_network':
				result = { 'status' : self.netcheck() }
			elif args[0] == 'create_backup':
				result = { 'status' : self.backupdir() }
			elif args[0] == 'latest_version':
				result = self.latestversion()
			elif args[0] == 'download_update':
				result = self.downloadupdate( args[1] )
			elif args[0] == 'setdir':
				result = self.set_listdir( args[1] if len( args ) == 2 else None )
			elif args[0] == 'chkdir':
				return { 'status' : "ok", 'result' : self.list_dir }
			elif args[0] == 'stitch_list':
				result = self.stitchlist()
			elif args[0] == 'image_space':
				result = self.imagespace();
			elif args[0] == 'update_thumbnail':
				result = self.thumbupdate( args[1], False )
			elif args[0] == 'overwrite_thumbnail':
				result = self.thumbupdate( args[1], True )
			elif args[0] == 'suggest_selection':
				result = self.suggestselect()
			elif args[0] == 'photoloom':
				result = self.photoloom_command( args[1:] )
			elif args[0] == 'backup':
				result = self.backup_command( args[1:] )
			elif args[0] == 'uploaddropbox':
				result = self.upload_dropbox( args[1], args[2] )
			elif args[0] == 'uploadpicasa':
				result = self.upload_picasa( args[1], args[2] )
			elif args[0] == 'evernoteurl':
				result = self.get_evernote_auth_url()
			elif args[0] == 'evernoteverifier':
				result = self.set_evernote_access_token( args[1] )
			elif args[0] == 'evernotelogout':
				result = self.evernote_logout()
			elif args[0] == 'evernoteisloggedin':
				result = self.evernote_is_logged_in()
			elif args[0] == 'uploadevernote':
				result = self.upload_evernote( args[1] )
			elif args[0] == 'checkexists':
				result = self.check_image_exists( args[1] )
			elif args[0] == 'checkstoryexists':
				result = self.check_story_exists( args[1] )
			elif args[0] == 'storydir':
				result = self.open_story_dir()
			elif args[0] == 'saverecord':
				result = self.save_record( args[1], args[2] )
			elif args[0] == 'saverotate':
				result = self.save_rotate( args[1], args[2], args[3] == 'true', args[4] == 'true', args[5] )
			elif args[0] == 'savecrop':
				result = self.save_crop( args[1], args[2], int( args[3] ), int( args[4] ), int( args[5] ), int( args[6] ), int( args[7] ))
			elif args[0] == 'autocolor':
				result = self.auto_color( args[1] )
			elif args[0] == 'adjustblueyellowbrightness':
				result = self.adjust_blue_yellow_brightness( args[1], args[2], args[3] )
			elif args[0] == 'savecoloradjust':
				result = self.adjust_color( args[1], args[2], args[3], args[4] )
			elif args[0] == 'cleanautocolorthumbs':
				result = self.clean_auto_color_thumbs( args[1] )
			elif args[0] == 'stitchthumbscanceled':
				result = self.stitch_thumbs_canceled( args[1] )
			else:
				result = { 'message' : "unrecognized command: %s" % args[0],
							'status' : "error" }
		except:
			result = { 'message' : "exception in command '%s'" % args[0],
						'status' : "error", 'traceback' : traceback.format_exc() }
		return result

	def startup( self ):
		return { 'status' : "ok" }

	def netcheck( self ):
		status = "error"
		try:
			conn = httplib.HTTPConnection( "www.google.com" )
			conn.request( "HEAD", "/" )
			r = conn.getresponse()
			if r.status == 200: status = "ok"
		except:
			pass
		return status

	def save_product_key( self, key ):
		path = join( self.rsrc_path, 'productkey' )
		try:
			if isfile( path ):
				remove( path )
			f = open( path, 'w' )
			print >>f, key
			return { 'status' : "ok" }
		except:
			return { 'status' : "error", 'message' : "Unable to save product key to %s" % path }

	def system_open( self, addr ):
		result = { 'status' : "error", 'message' : "could not open %s" % addr }
		try:
			if is_mac():
				os.system( "open " + "\"" + addr + "\"" )
			elif is_windows():
				if addr.startswith( "http" ):
					import webbrowser
					webbrowser.open( addr )
				else:
					from ctypes import windll
					windll.shell32.ShellExecuteW( None, u'explore',
						unicode( addr ), None, None, 1 );
			result = { 'status' : "ok" }
		except:
			pass
		return result

	def resource_open( self, name ):
		fpath = join( self.rsrc_path, name )
		try:
			if is_windows():
				from ctypes import windll
				result = windll.shell32.ShellExecuteW( None, u'open',
					unicode( fpath ), None, None, 1 );
				if result > 32:
					return { 'status' : "ok" }
				else:
					return { 'status' : "error", 'code' : result }
			elif is_mac():
				result = os.system( "open " + "\"" + fpath + "\"" )
				if result == 0:
					return { 'status' : "ok" }
				else:
					return { 'status' : "error", 'code' : result }
		except:
			pass
		return { 'status' : "error", 'message' : "unknown" }

	def flip_open( self ):
		[last_mount, chkpath] = self.access_flip_path()
		if chkpath and isdir( chkpath ):
			self.system_open( chkpath )
			return { 'status' : "ok" }
		else:
			return { 'status' : "error", 'message' : "No path the Flip-Pal images" }

	def latestversion( self ):
		try:
			stream = urlopen( "http://couragent.net/toolbox/update/version/" )
			self.update = json.loads( stream.read())
			return { 'status' : "ok", 'version' : self.update['full']['version'] }
		except:
			result = { 'status' : "error", 'traceback' : traceback.format_exc() }
		return result

	def downloadupdate( self, current ):
		if not self.update:
			return { 'status' : "ok", 'message' : "no update path, run latest_version first" }
		try:
			deltas = self.update['delta']
			updater = None
			size = None
			for d in deltas:
				if d['from'] == current:
					updater = d['updater']
					size = d['size']
					break
			if updater and size:
				location = join( self.rsrc_path, basename( updater ))
				if isfile( location ):
					os.remove( location )
				urlretrieve( updater, location )
				if not isfile( location ):
					return { 'status' : "error", 'message' : "updater could not be downloaded" }
				if not getsize( location ) == int( size ):
					return { 'status' : "error", 'message' : "download incorrect size: %s" % size }
				return { 'status' : "ok", 'version' : self.update['full']['version'],
						'path' : location }
		except:
			result = { 'status' : "error", 'traceback' : traceback.format_exc() }
		return result

	def calcspace( self, flist ):
		numbytes = 0
		for f in flist:
			if isfile( f ):
				sinfo = os.stat( f )
				fbytes = sinfo.st_size
				numbytes += fbytes
		mb = round( numbytes / 1024 / 1024 )
		return "%d MB" % int( mb ) # result in MB

	def imagespace( self ):
		flist = self.stitchlist()['result']
		return { 'status' : "ok", 'result' :
				{ 'count' : len( flist ), 'size' : self.calcspace( flist ) }}

	def list_dir_images( self, path, result ):
		if isdir( path ):
			for f in os.listdir( path ):
				if not f.startswith( "." ) and ( fnmatch( f, "*.JPG" ) or fnmatch( f, "*.jpg" )):
					# Don't use join(), result going back in JSON, escape confuses things
					result.append( "%s/%s" % ( path, f ))

	def photoloom_command( self, args ):
		if not self.photoloom:
			import photoloom
			self.photoloom = photoloom.PhotoloomCommand( self.input, self.output, self.rsrc_path )
		return self.photoloom.run_command( args )

	def backup_command( self, args ):
		if not self.backup:
			import fpbackup
			self.backup = fpbackup.BackupCommand( self.input, self.output, self.rsrc_path )
		return self.backup.run_command( args )

	# Returns [last_mount, path] where last_mount is the volume path that was stored
	# previously on a successful read exists. If it's None then the volumes had to be
	# scanned to obtain it. The path value is the path to the 100_SCAN directory.

	def access_flip_path( self ):
		if is_windows():
			from ctypes import windll
			windll.kernel32.SetErrorMode(1) # disable drive error dialog
		last_mount = None
		chkpath = None
		self.last_mount_path = join( self.rsrc_path, "last_mount.txt" )
		if isfile( self.last_mount_path ):
			try:
				fp = open( self.last_mount_path )
				# Previously scanned drive, shouldn't change often
				last_mount = fp.read()
			except IOError:
				pass
			finally:
				if fp: fp.close()
			if is_windows():
				chkpath = join( last_mount, self.vol_dir )
			elif is_mac():
				chkpath = join( last_mount, self.vol_dir[1:] )
			if not isdir( chkpath ):
				# Drive letter of SD card changed since saved
				last_mount = None
				os.unlink( self.last_mount_path )
		return [last_mount, chkpath]

	def set_listdir( self, path ):
		self.list_dir = path;
		return { 'status' : "ok" }

	def stitchlist( self ):
		result = []
		if self.list_dir and isdir( self.list_dir ):
			# First try a directory manually chosen by user with setdir.
			self.list_dir_images( self.list_dir, result )
		else:
			# If not set, try default Flip-Pal image directory.
			[last_mount, chkpath] = self.access_flip_path()
			if last_mount != None:
				self.list_dir_images( chkpath, result )
			else:
				result = self.os_stitch_list()
		if len( result ) > 0:
			result.sort()
		return { 'status' : "ok", 'result' : result }

	# Currently unused
	def suggestselect( self ):
		stitch_list = self.stitchlist()
		paths = stitch_list['result']
		result = []
		return { 'status' : "ok", 'result' : result }

	def os_stitch_list( self ):
		if is_windows():
			return self.win_stitchlist()
		elif is_mac():
			return self.mac_stitchlist()

	def mac_stitchlist( self ):
		import re
		# Mac, spawn df(1) command
		result = []
		out = run_process( "df" )
		for line in out.splitlines():
			m = re.search( ".+(\/Volumes\/.+)$", line )
			if m:
				# "/Volumes/FLIP-PAL", "DCIM/100_SCAN"
				chkpath = join( m.group( 1 ), self.vol_dir[1:] )
				self.list_dir_images( chkpath, result )
				if len(result) > 0:
					try:
						fp = open( self.last_mount_path, "w" )
						if fp: fp.write( m.group( 1 ))
					finally:
						if fp: fp.close()
					break
		return result


	def win_stitchlist( self ):
		drives = []
		from ctypes import windll
		bitmask = windll.kernel32.GetLogicalDrives()
		for letter in string.uppercase:
			# 2 from GetDriveTypeW() is a removable drive
			# 3 from GetDriveTypeW() is a fixed drive
			drive_type = windll.kernel32.GetDriveTypeW( u"%s:\\" % letter )
			if bitmask & 1 and (drive_type == 2 or drive_type == 3):
				drives.append( letter )
			bitmask >>= 1
		result = []
		for drive in drives:
			try:
				self.list_dir_images( join( "%s:" % drive, self.vol_dir ), result )
			except:
				pass
			if len( result ) > 0:
				fp = None
				try:
					fp = open( self.last_mount_path, "w" )
					if fp: fp.write( "%s:" % drive )
				finally:
					if fp: fp.close()
				break
		return result

	def thumbupdate( self, image, overwrite ):
		tdir = join( dirname( image ), ".thumb" )
		if not isdir( tdir ): os.mkdir( tdir )
		tpath = join( tdir, basename( image ))
		cmd = ""
		if(( not isfile( tpath )) or ( getmtime( image ) > getmtime( tpath )) or overwrite ):
			# Thumbnail missing or needs to be updated
			try:
				convert = join( self.rsrc_path, "util", "convert" ) + ( ".exe" if is_windows() else "" )
				cmd = "\"%s\" -verbose -resize 300x300 \"%s\" \"%s\"" % ( convert, image, tpath )
				line = run_process( cmd )
				return { 'status' : "ok", 'thumbnail_path' : tpath.replace( "\\", "/" ),
						'size' : line.split()[2] }
			except:
				return { 'status' : "error", 'message' : "exception executing command '%s'" % cmd,
						'traceback' : traceback.format_exc() }
		else:
			# Existing thumbnail good, just need info
			try:
				jhead = join( self.rsrc_path, "util", "jhead" ) + ( ".exe" if is_windows() else "" )
				cmd = "\"%s\" \"%s\"" % ( jhead, tpath )
				res = parseresolution( cmd )
				return { 'status' : "ok", 'thumbnail_path' : tpath.replace( "\\", "/" ),
						'size' : "%sx%s" % ( res[0], res[1] )}
			except:
				return { 'status' : "error", 'message' : "exception executing command '%s'" % cmd,
							'traceback' : traceback.format_exc() }

	def upload_dropbox( self, filepath, token ):
		filepath = normpath( filepath )
		try:
			filename, data = get_image( filepath )
			url = 'https://api-content.dropbox.com/1/files_put/auto/' + filename
			headers = { 'Authorization' : 'Bearer ' + token, 'Content-Type' : 'application/octet-stream', 'Content-Length' : str( len( data ) ) }
			upload_request( url, data, headers, True )
			return { 'status' : 'ok' }
		except:
			return { 'status' : "error", 'message' : "exception uploading to Dropbox",
					'traceback' : traceback.format_exc() }

	def upload_picasa( self, filepath, token ):
		filepath = normpath( filepath )
		try:
			filename, data = get_image( filepath )
			url = 'https://picasaweb.google.com/data/feed/api/user/default/albumid/default'
			data, boundary = picasa_get_post_data( data, filename )
			headers = { 'Authorization' : 'Bearer ' + token, 'Content-Type' : 'multipart/related;boundary=%s' % boundary, 'Slug' : filename, 'MIME-version' : '1.0', 'Content-Length' : str( len( data ) ) }
			upload_request( url, data, headers, False )
			return { 'status' : 'ok' }
		except:
			tb = traceback.format_exc()
			if tb.find( 'HTTP Error 201: Created' ) != -1:
				# exception is raised as pseudo-return value indicating success
				return { 'status' : 'ok' }
			return { 'status' : "error", 'message' : "exception uploading to Picasa",
					'traceback' : tb }

	def get_evernote_auth_url( self ):
		self.evernote_uploader = EvernoteUploader( False )
		return { 'status' : 'ok', 'url' : self.evernote_uploader.authorize_url }

	def set_evernote_access_token( self, verifier ):
		if len( verifier ) == 0:
			self.evernote_uploader = None
			return { 'status' : "error", 'message' : "Evernote verifier cannot be length 0",
					'traceback' : "" }
		try:
			self.evernote_uploader.set_access_token( verifier )
			return { 'status' : 'ok' }
		except:
			self.evernote_uploader = None
			return { 'status' : "error", 'message' : "exception setting Evernote access token",
					'traceback' : traceback.format_exc() }

	def evernote_logout( self ):
		self.evernote_uploader = None
		return { 'status' : 'ok' }

	def evernote_is_logged_in( self ):
		loggedin = self.evernote_uploader is not None
		return { 'status' : 'ok', 'loggedin' : str( loggedin ) }

	def upload_evernote( self, filepath ):
		filepath = normpath( filepath )
		try:
			filename, data = get_image( filepath )
			self.evernote_uploader.upload_image( filename, data )
			return { 'status' : 'ok' }
		except:
			return { 'status' : "error", 'message' : "exception uploading to Evernote",
					'traceback' : traceback.format_exc() }

	def check_image_exists( self, thumbpath ):
		path = thumbpath_to_imagepath( thumbpath )
		return { 'status' : 'ok', 'file_exists' : str( isfile( path ) ) }

	def check_story_exists( self, thumbpath ):
		path = thumbpath_to_imagepath( thumbpath )
		path = join( dirname( path ), STORY_SUBDIR, basename( path ) )
		return { 'status' : 'ok', 'file_exists' : str( isfile( path ) ) }

	def stitch_thumbs_canceled( self, arg ):
		if arg == '-1':
			return { 'status' : 'ok', 'canceled' : 'true' if self.stitchthumbscanceled else 'false' }
		if arg == '0':
			self.stitchthumbscanceled = False
			return { 'status' : 'ok' }
		if arg == '1':
			self.stitchthumbscanceled = True
			return { 'status' : 'ok' }
		return { 'status' : 'error', 'message' : "invalid argument '%s'" % arg }

	def open_story_dir( self ):
		if self.list_dir and isdir( self.list_dir ):
			d = self.list_dir
		else:
			[last_mount, chkpath] = self.access_flip_path()
			if isdir( chkpath ):
				d = chkpath
			else:
				d = None
		if d:
			storydir = join( normpath( d ), STORY_SUBDIR )
			if not isdir( storydir ):
				mkdir( storydir )
			if isdir( storydir ):
				return self.system_open( storydir )
			else:
				errmsg = "Failed to create directory: {}".format( storydir )
				return { 'status' : "error", 'message' : errmsg }
		else:
			return { 'status' : "error", 'message' : "Couldn't find {} directory".format( STORY_SUBDIR ) }


	def save_record( self, thumbpath, filename ):
		thumbpath = normpath( thumbpath )
		imgpath = thumbpath_to_imagepath( thumbpath )

		imgdir = join( dirname( imgpath ), STORY_SUBDIR )
		if not isdir( imgdir ):
			mkdir( imgdir )

		existing_savepath = join( imgdir, filename )
		overwrite = isfile( existing_savepath)
		# if we're overwriting something, save as a temporary file first, then overwrite at the end
		savepath = join( imgdir, "new.%s" % filename )

		if isfile( savepath ):
			return { 'status' : 'error', 'message' : "Can't create temporary file '%s' - file already exists" % savepath, 'traceback' : '' }

		#convert ~/Downloads/testimages/leopard0.jpg -resize 1920x1080\> -gravity center -background black -extent 1920x1080 leopard0_2.jpg
		convert = join( self.rsrc_path, "util", "convert" ) + ( ".exe" if is_windows() else "" )
		resized_imgpath = join( imgdir, "%s.%s" % ( randid(), basename( imgpath ) ) )
		cmd = "\"%s\" \"%s\" -resize 1920x1080 -gravity center -background black -extent 1920x1080 \"%s\"" % ( convert, imgpath, resized_imgpath )
		try:
			line = run_process( cmd )
		except:
			if isfile( resized_imgpath ):
				remove( resized_imgpath )
			return { 'status' : 'error', 'message' : "exception executing command '%s'" % cmd,
					'traceback' : traceback.format_exc() }

		ffmpeg = join( self.rsrc_path, "util", "ffmpeg" ) + ( ".exe" if is_windows() else "" )
		rawaudio = join( self.rsrc_path, "recording.raw" )
		cmd = "\"%s\" -loop 1 -i \"%s\" -f f32le -ac 2 -ar 44100 -i \"%s\" -shortest -vcodec mpeg4 -acodec aac -strict -2 -b:a 240k -r 1 \"%s\"" % ( ffmpeg, resized_imgpath, rawaudio, savepath )
		try:
			line = run_process( cmd )
			if overwrite:
				remove( existing_savepath )
			# remove the .new extension
			rename( savepath, existing_savepath )
			ret = { 'status' : 'ok' }
		except:
			ret = { 'status' : 'error', 'message' : "exception executing command '%s'" % cmd,
					'traceback' : traceback.format_exc() }
		remove( resized_imgpath )
		return ret

	def save_rotate( self, thumbpath, filename, flipx, flipy, angle ):
		# saverotate C:/DCIM/100_SCAN/.thumb/square.jpg square2.jpg false true 0

		thumbpath = normpath( thumbpath )
		# path to image to be modified
		imgpath = thumbpath_to_imagepath( thumbpath )
		# path to save location for new image
		savepath = join( dirname( imgpath ), filename )
		overwrite = isfile( savepath )
		# if we're overwriting something, save as a temporary file first, then overwrite at the end
		savepath = "%s.new" % savepath

		if isfile( savepath ):
			return { 'status' : 'error', 'message' : "Can't create temporary file '%s' - file already exists" % savepath, 'traceback' : '' }

		convert = join( self.rsrc_path, "util", "convert" ) + ( ".exe" if is_windows() else "" )
		cmd = "\"%s\" \"%s\" -verbose" % ( convert, imgpath )
		if flipx:
			cmd += ' -flop'
		if flipy:
			cmd += ' -flip'
		cmd += ' -rotate %s' % angle
		cmd += " \"%s\"" % savepath
		try:
			line = run_process( cmd )
			if overwrite:
				remove( savepath[:-4] )
			# remove the .new extension
			rename( savepath, savepath[:-4] )
			return { 'status' : 'ok' }
		except:
			return { 'status' : 'error', 'message' : "exception executing command '%s'" % cmd,
					'traceback' : traceback.format_exc() }

	def save_crop( self, thumbpath, filename, x1, y1, x2, y2, thumbwidth ):
		# savecrop C:/DCIM/100_SCAN/.thumb/SCAN0067.JPG SCAN0067_.JPG 50 50 150 250 200 300

		thumbpath = normpath( thumbpath )
		# path to image to be modified
		imgpath = thumbpath_to_imagepath( thumbpath )
		# path to save location for new image
		savepath = join( dirname( imgpath ), filename )
		overwrite = isfile( savepath )
		# if we're overwriting something, save as a temporary file first, then overwrite at the end
		savepath = "%s.new" % savepath

		if isfile( savepath ):
			return { 'status' : 'error', 'message' : "Can't create temporary file '%s' - file already exists" % savepath, 'traceback' : '' }

		jhead = join( self.rsrc_path, "util", "jhead" ) + ( ".exe" if is_windows() else "" )
		cmd = "\"%s\" \"%s\"" % ( jhead, imgpath )
		res = parseresolution( cmd )

		scale = float(res[0]) / thumbwidth
		coords = (x1 * scale, y1 * scale)
		width = (x2 - x1) * scale
		height = (y2 - y1) * scale

		convert = join( self.rsrc_path, "util", "convert" ) + ( ".exe" if is_windows() else "" )
		cmd = "\"%s\" \"%s\" -crop %sx%s+%s+%s +repage \"%s\"" % ( convert, imgpath, width, height, coords[0], coords[1], savepath )
		try:
			line = run_process( cmd )
			if overwrite:
				remove( savepath[:-4] )
			# remove the .new extension
			rename( savepath, savepath[:-4] )
			return { 'status' : 'ok' }
		except:
			return { 'status' : 'error', 'message' : "exception executing command '%s'" % cmd,
					'traceback' : traceback.format_exc() }

	def do_auto_color( self, rsrc, convert, imgpath, use_cached_means=False ):
		savepath = normpath( join( dirname( imgpath ), 'auto_%s' % basename( imgpath ) ) )

		if isfile( savepath ):
			# if auto-adjusted image exists, overwrite it
			remove( savepath )

		mpc_files = tmp_filenames( rsrc )

		cmd = "\"%s\" \"%s\" +repage \"%s\"" % ( convert, imgpath, mpc_files['1'] )
		run_process( cmd )

		rratio = self.rratio
		gratio = self.gratio
		bratio = self.bratio

		if not use_cached_means:
			separate_channels( convert, mpc_files )

			cmd = "\"%s\" \"%s\" -set colorspace RGB -colorspace Rec601Luma \"%s\"" % ( convert, mpc_files['1'], mpc_files['L'] )
			run_process( cmd )

			ngray = get_channel_mean( convert, mpc_files['L'] )
			rratio = get_channel_mean_ratio( convert, mpc_files['R'], ngray )
			gratio = get_channel_mean_ratio( convert, mpc_files['G'], ngray )
			bratio = get_channel_mean_ratio( convert, mpc_files['B'], ngray )

			self.rratio = rratio
			self.gratio = gratio
			self.bratio = bratio

		cmd = "\"%s\" \"%s\" -color-matrix \"%s 0 0 0 %s 0 0 0 %s\" \"%s\"" % ( convert, mpc_files['1'], rratio, gratio, bratio, mpc_files['1'] )
		run_process( cmd )

		separate_channels( convert, mpc_files )
		clip_combine_channels_separate( convert, mpc_files, savepath )

		remove_tmp_files( mpc_files )
		return savepath

	def auto_color( self, imgpath ):
		# autocolor C:/DCIM/100_SCAN/SCAN0047.JPG
		convert = join( self.rsrc_path, "util", "convert" ) + ( ".exe" if is_windows() else "" )
		try:
			path = self.do_auto_color( self.rsrc_path, convert, imgpath )
			return { 'status' : 'ok', 'path' : path }
		except:
			return { 'status' : 'error', 'message' : 'exception auto-adjusting color',
					'traceback' : traceback.format_exc() }

	def adjust_blue_yellow_brightness( self, imgpath, percentcolor, percentbrightness ):
		convert = join( self.rsrc_path, "util", "convert" ) + ( ".exe" if is_windows() else "" )
		try:
			path = do_adjust_blue_yellow_brightness( convert, imgpath, percentcolor, percentbrightness )
			return { 'status' : 'ok', 'path' : path }
		except:
			return { 'status' : 'error', 'message' : 'exception adjusting blue/yellow and brightness',
					'traceback' : traceback.format_exc() }

	def adjust_color( self, thumbpath, filename, brightness, color ):
		# savecoloradjust C:/DCIM/100_SCAN/.thumb/baseballteam.jpg color_baseballteam.jpg 15 -15
		thumbbase = basename( thumbpath )
		if len(thumbbase) > 5 and thumbbase[0:5] == 'auto_':
			# When called from Flip-Pal Toolbox, the active thumbnail is an auto-colored
			# image, so it will have the auto_ prefix. Delete this prefix to get the
			# actual image path.
			thumbpath = join( dirname( thumbpath ), thumbbase[5:] )
		convert = join( self.rsrc_path, "util", "convert" ) + ( ".exe" if is_windows() else "" )
		imgpath = thumbpath_to_imagepath( thumbpath )
		# path to save location for new image
		savepath = join( dirname( imgpath ), filename )
		overwrite = isfile( savepath )
		savepath = "%s.new" % savepath

		if isfile( savepath ):
			return { 'status' : 'error', 'message' : "Can't create temporary file '%s' - file already exists" % savepath, 'traceback' : '' }

		try:
			# We can use cached ratios here because this is only called after preview has
			# been generated.
			autocolorpath = self.do_auto_color( self.rsrc_path, convert, imgpath, True )
			adjustedpath = do_adjust_blue_yellow_brightness( convert, autocolorpath, color, brightness )
			remove( autocolorpath )
			rename( adjustedpath, savepath )
			if overwrite:
				remove( savepath[:-4] )
			# remove the .new extension
			rename( savepath, savepath[:-4] )
			return { 'status' : 'ok' }
		except:
			return { 'status' : 'error', 'message' : 'exception adjusting color',
					'traceback' : traceback.format_exc() }

	def clean_auto_color_thumbs( self, thumbpath ):
		# thumbpath is any thumbnail in the .thumb directory
		thumbdir = dirname( thumbpath )
		files = [ f for f in listdir( thumbdir ) if isfile( join( thumbdir, f ) ) ]
		for f in files:
			if is_autogenerated_thumb( f ):
				remove( join( thumbdir, f ) )
		return { 'status' : 'ok' }

def is_autogenerated_thumb( filename ):
	return filename[0:5] == 'auto_' or filename[0:6] == '_auto_'

def do_adjust_blue_yellow_brightness( convert, imgpath, percentcolor, percentbrightness ):
	savepath = normpath( join( dirname( imgpath ), '_%s' % basename( imgpath ) ) )
	op = get_imagemagick_tint_op( percentcolor )
	cmd = "\"%s\" \"%s\" -channel Y -evaluate %s \"%s\"" % ( convert, imgpath, op, savepath )
	run_process( cmd )
	cmd = "\"%s\" \"%s\" -brightness-contrast %s \"%s\"" % ( convert, savepath, percentbrightness, savepath )
	run_process( cmd )
	return savepath

def get_imagemagick_tint_op( percent ):
	op = 'Subtract'
	if percent[0] == '-':
		op = 'Add'
		percent = percent[1:]
	return "%s %s%%" % ( op, percent )

def randid( size=5, chars=ascii_uppercase + digits ):
   return ''.join( choice( chars ) for _ in range( size ) )

def tmp_filenames( rsrc ):
	mpc_files = {}
	mpc_files['1'] = normpath( join( rsrc, '%s_1.mpc' % randid() ) )
	mpc_files['R'] = normpath( join( rsrc, '%s_R.mpc' % randid() ) )
	mpc_files['G'] = normpath( join( rsrc, '%s_G.mpc' % randid() ) )
	mpc_files['B'] = normpath( join( rsrc, '%s_B.mpc' % randid() ) )
	mpc_files['L'] = normpath( join( rsrc, '%s_L.mpc' % randid() ) )
	return mpc_files

def remove_tmp_files( mpc_files ):
	for k in mpc_files:
		if( isfile( mpc_files[k] ) ):
			remove( mpc_files[k] )
			remove( '%s.cache' % mpc_files[k][:-4] )

def separate_channels( convertpath, mpc_files ):
	separate_cmd_template = "\"%s\" \"%s\" -set colorspace RGB -channel %s -separate \"%s\""
	cmd = separate_cmd_template % ( convertpath, mpc_files['1'], 'R', mpc_files['R'] )
	run_process( cmd )
	cmd = separate_cmd_template % ( convertpath, mpc_files['1'], 'G', mpc_files['G'] )
	run_process( cmd )
	cmd = separate_cmd_template % ( convertpath, mpc_files['1'], 'B', mpc_files['B'] )
	run_process( cmd )

def calc_mean_ratio( convertpath, ngray, mean ):
	cmd = "\"%s\" xc: -format \"%%[fx:%s/%s]\" info:" % ( convertpath, ngray, mean )
	return run_process( cmd )

def get_channel_mean( convertpath, imgpath ):
	cmd = "\"%s\" \"%s\" -format \"%%[mean]\" info:" % ( convertpath, imgpath )
	mean = ""
	mean = run_process( cmd )
	cmd = "\"%s\" xc: -format \"%%[fx:100*%s/quantumrange]\" info:" % ( convertpath, mean )
	return run_process( cmd )

def get_channel_mean_ratio( convertpath, imgpath, ngray ):
	mean = get_channel_mean( convertpath, imgpath )
	if mean == '0' or mean == '0.0': mean = '100'
	return calc_mean_ratio( convertpath, ngray, mean )

def clip_combine_channels_together( convertpath, mpc_files, savepath ):
	cmd = "\"%s\" \"%s\" \"%s\" \"%s\" -combine -colorspace RGB \"%s\"" % ( convertpath, mpc_files['R'], mpc_files['G'], mpc_files['B'] , mpc_files['1'] )
	run_process( cmd )
	cmd = "\"%s\" \"%s\" -contrast-stretch 0.1%%,0.1%% \"%s\"" % ( convertpath, mpc_files['1'], savepath )
	run_process( cmd )

def clip_combine_channels_separate( convertpath, mpc_files, savepath ):
	cmd_template = "\"%s\" \"%s\" -contrast-stretch 0.1%%,0.1%% \"%s\""
	cmd = cmd_template % ( convertpath, mpc_files['R'], mpc_files['R'] )
	run_process( cmd )
	cmd = cmd_template % ( convertpath, mpc_files['G'], mpc_files['G'] )
	run_process( cmd )
	cmd = cmd_template % ( convertpath, mpc_files['B'], mpc_files['B'] )
	run_process( cmd )
	cmd = "\"%s\" \"%s\" \"%s\" \"%s\" -combine -colorspace RGB \"%s\"" % ( convertpath, mpc_files['R'], mpc_files['G'], mpc_files['B'], savepath )
	run_process( cmd )

def thumbpath_to_imagepath( thumbpath ):
	base = basename( thumbpath )
	imgpath = join( dirname( thumbpath ), '..', base )
	return normpath( imgpath )

def parseresolution( cmd ):
	out = run_process( cmd )
	lines = out.splitlines()
	resLine = None
	for line in lines:
		if line.startswith( "Resolution" ):
			resLine = line
			break
	pts = resLine.split() # Resolution    : 600 x 400
	return ( pts[2], pts[4] )

