// Flip-Pal Toolbox file archival function
// 2013 Jan 17 - Jim Moy, jim.moy@couragent.net

// Top-level function
function clearCard() {
	$('#copy-dialog').bind( 'shown', function() {
		$('#copy-dialog').unbind( 'shown' );
		commandLine( ['stitch_list'], 'copyImagesCallback' );
	});
	window.flipCopyCanceled = false;
	$('#progress_prompt').html( "Copying and verifying images..." );
	$('#cancel-or-close').text( "Cancel" );
	$('#cancel-or-close').click( function( event ) {
	$('#progress-bar').css( 'width', "0%" );
	$('#dialogbody').show();
		cancelCloseButtonClicked();
	});
	// Show the copy dialog, proceed to copyImagesCallback() with file list
	$('#copy-dialog').modal({ show:true, backdrop:'static', keyboard:false });
}

function cancelCloseButtonClicked() {
	if( $('#cancel-or-close').text() == "Cancel" ) {
		window.flipCopyCanceled = true;
	} else if( $('#cancel-or-close').text() == "Close" ) {
		$('#copy-dialog').modal( 'hide' );
		commandLine(['backup', 'abort'], '' );
	}
}

function copyImagesCallback( arg ) {
	if( checkMoveCancel() )
		return;
	var obj = $.parseJSON( arg )
	if( obj.status == "ok" && obj.result.length > 0 ) {
		window.flipCopyList = obj.result;
		// Setup progress indicator, break bar into this many segments.
		// One increment per copy, verify, and the last one is the removal
		// of files from the SD card after the copy/verify is ok.
		window.flipIncrement = 100.0 / ( obj.result.length * 2 + 1 );
		window.flipProgress = 0.0;
		// Creates the backup directory
		commandLine( ['backup', 'start'], 'imagesDirectoryCallback' );
	} else {
		showError( $('#str-move-list-fail').text() );
	}
}

function imagesDirectoryCallback( arg ) {
	if( checkMoveCancel() )
		return;
	var obj = $.parseJSON( arg )
	if( obj.status == "ok" && obj.directory ) {
		window.flipCopyDirectory = obj.directory;
		// Not recursive, but sort of a game of ping-pong, alternately copying
		// then verifying a file.
		copyFileAtIndex( 0 );
	} else {
		showError( $('#str-move-create-fail').text() );
	}
}

function copyFileAtIndex( index ) {
	window.flipCopyIndex = index;
	commandLine(['backup', 'copy', window.flipCopyList[index]], 'copyFileCallback' );
}

function copyFileCallback( arg ) {
	if( checkMoveCancel() )
		return;
	var obj = $.parseJSON( arg )
	var origpath = window.flipCopyList[window.flipCopyIndex];
	if( obj.status == "ok" ) {
		incrementProgress();
		var destpath = obj.result;
		commandLine(['backup', 'verify', destpath, origpath], 'verifyFileCallback' );
	} else {
		var dest = window.flipCopyDirectory;
		var fname = baseName( origpath );
		showError( String.format( $('#str-copy-fail').text(), fname, dest ));
	}
}

function verifyFileCallback( arg ) {
	if( checkMoveCancel() )
		return;
	var obj = $.parseJSON( arg )
	if( obj.status == "ok" ) {
		incrementProgress();
		if( window.flipCopyIndex == ( window.flipCopyList.length - 1 )) {
			// Verified last file, done copying and verifying, now clean up.
			// Last increment taken up by deletion of all the files.
			window.flipIncrement /= window.flipCopyList.length;
			deleteFileAtIndex( 0 );
		} else {
			// Go to the next file
			copyFileAtIndex( window.flipCopyIndex + 1 );
		}
	} else {
		showError( $('#str-verify-fail').text() );
	}
}

function deleteFileAtIndex( index ) {
	window.flipDeleteIndex = index;
	var path = window.flipCopyList[window.flipDeleteIndex];
	commandLine( ['backup', 'delete', path], 'deleteFileCallback' );
}

function deleteFileCallback( arg ) {
	var obj = $.parseJSON( arg );
	incrementProgress();
	if( obj.status == "ok" ) {
		if( window.flipDeleteIndex == ( window.flipCopyList.length - 1 )) {
			$('#modalLabel').html( $('#str-move-done').text() );
			$('#progress_prompt').html( $('#str-move-done-prompt').text() );
			$('#dialogbody').hide();
			$('#dialog-footer-btns').hide( 'fast' );
			$('#dialogend').show( 'fast' );
		} else {
			deleteFileAtIndex( window.flipDeleteIndex + 1 );
		}
	} else {
		showError( $('#str-delete-fail').text() );
	}
}

function showError( errMsg ) {
	$('#progress_prompt').html( errMsg );
	$('#cancel-or-close').html( "Close" );
	$('#dialogbody').hide();
}

function checkMoveCancel() {
	if( window.flipCopyCanceled ) {
		$('#progress_prompt').html( $('#str-copy-cancel').text() );
		$('#cancel-or-close').html( "Close" );
		$('#dialogbody').hide();
		return true;
	}
	return false;
}

function incrementProgress() {
	window.flipProgress += window.flipIncrement;
	$('#progress-bar').css( 'width',  Math.round( window.flipProgress ).toString() + "%" );
}

