import evernote.edam.type.ttypes as Type
import hashlib

from evernote.api.client import *

CONSUMER_KEY = 'am-couragent'
CONSUMER_SECRET = '0760c178fd6b9ff8'
CALLBACK_URL = 'https://www.evernote.com/flippaltoolbox_login_success'

class EvernoteUploadError( Exception ):
	pass

def md5_hexdigest( data ):
	md5 = hashlib.md5()
	md5.update( data )
	return md5.hexdigest()

def make_resource_data( rawdata, rawdatahash ):
	data = Type.Data()
	data.body = rawdata
	data.size = len( rawdata )
	data.bodyHash = rawdatahash
	return data

def make_image_resource( filename, rawdata, rawdatahash ):
	resattr = Type.ResourceAttributes()
	resattr.fileName = filename
	#resattr.attachment = False
	data = make_resource_data( rawdata, rawdatahash )
	res = Type.Resource()
	res.data = data
	res.mime = 'image/jpeg'
	res.attributes = resattr
	return res

def make_note_attributes():
	noteattrs = Type.NoteAttributes()
	noteattrs.author = 'Flip-Pal Toolbox'
	return noteattrs

def make_image_note( filename, data ):
	datahash = md5_hexdigest( data )
	res = make_image_resource( filename, data, datahash )
	noteattrs = make_note_attributes()
	note = Type.Note()
	note.title = filename
	note.resources = [res]
	note.attributes = noteattrs
	note.content = '<?xml version="1.0" encoding="UTF-8"?><!DOCTYPE en-note SYSTEM "http://xml.evernote.com/pub/enml2.dtd">'
	link = '<en-media type="image/jpeg" hash="%s" />' % datahash
	note.content += '<en-note>%s</en-note>' % link
	return note

class EvernoteUploader:
	''' Manage authorization and authentication for Evernote connection and
	simplify image uploads. set_access_token should only be called once for each
	instance '''
	def __init__( self, sandbox ):
		self.sandbox = sandbox
		self.client = EvernoteClient(
				consumer_key=CONSUMER_KEY,
				consumer_secret=CONSUMER_SECRET,
				sandbox=self.sandbox )
		self.request_token = self.client.get_request_token( CALLBACK_URL )
		self.authorize_url = self.client.get_authorize_url( self.request_token )
		self.access_token = None
	def set_access_token( self, oauth_verifier ):
		if self.access_token is None:
			self.access_token = self.client.get_access_token(
					self.request_token['oauth_token'],
					self.request_token['oauth_token_secret'],
					oauth_verifier )
			self.client = EvernoteClient( token=self.access_token )
		else:
			raise EvernoteUploadError( 'Redundantly set access token' )
	def upload_image( self, filename, filecontents, ):
		note = make_image_note( filename, filecontents )
		note_store = self.client.get_note_store()
		return note_store.createNote( note )

#en = EvernoteUploader( True )
#print( en.authorize_url )
#verifier = raw_input('oauth_verifier:')
#en.set_access_token( verifier )
#upload_image_file( 'C:\\DCIM\\100_SCAN\\turtle.jpg', en )

